/*
 *  btgconst.h      Header file for btree.c and isam.c
 *
 *
 *              There are detailed comments above all the paramaters that
 *              can or should be tailored. Please take a few minutes and
 *              read through this file; all troubles getting the package
 *              working initially are due to one or more of these #defines
 *              being incorrectly set.
 *
 *                  fcntl.h         Some compilers require this header be
 *                                  #included for file access routines. 
 *                                  If yours is not one of them, comment out
 *                                  the #include.
 *
 *                  CREATMODE       The argument passed to creat()
 *
 *                  RWMODE          The argument passed to open()
 *          
 *                  voids           If your compiler does NOT support voids,
 *                                  uncomment the 'void' #define below.
 *
 *                  MAXKEYSIZE      The maximum key length. Don't
 *                                  forget the trailing NULL and the
 *                                  extra 3 byte overhead per key for
 *                                  duplicates.
 *
 *                  NUMINDEXS       The maximum number of index files that
 *                                  can be open at one time.
 *
 */

/*
 *  fcntl.h
 *
 *  Some compilers have a header file that must be included for file
 *  input/output functions. If your compiler comes with a fcntl.h file,
 *  then uncomment this #include
 */
#include    <fcntl.h>
#include    <memory.h>
/*
 *  Some common CREATMODE and RWMODE values
 *  ---- ------ --------- --- ------ ------
 *
 *  Wizard C            CREATMODE  == 0200
 *                      RWMODE     == O_RDWR
 *
 *  MicroSoft v3.x      CREATMODE  == 0200
 *                      RWMODE     == (O_RDWR | O_BINARY)
 *                      (sign extends chars)
 *
 *  Hi-Tech C           CREATMODE  == 0200   (not really critical)
 *                      RWMODE     == 2
 *                      (doesn't require fcntl.h #include)
 *
 *  Lattice 3.0+        CREATMODE == (0x8002 | S_IWRITE | S_IREAD)
 *                      RWMODE    == 0x8002
 *
 *  Mark Williams C     CREATMODE == 0200 (doesn't matter)
 *                      RWMODE    == 2
 *
 *  DataLight C         CREATMODE  == 0
 *                      RWMODE     == 2
 *
 *  CWare DeSmet C      CREATMODE  == 0644
 *                      RWMODE     == 2
 *                      (sign extends chars)
 *
 *  Lattice < 3.0       CREATMODE  == 0x8644
 *                      RWMODE     == 0x8002
 *
 *  C.I. C86            CREATMODE  == BUPDATE
 *                      RWMODE     == BUPDATE
 *
 *  unix/xenix          CREATMODE  == 0644
 *                      RWMODE     == 2
 */
#define    CREATMODE    0666
#define    RWMODE       (O_RDWR)

/*
 *  Voids
 *  -----
 *  if your compiler supports void types, comment out the void #define.
 *  Most compilers now do.
 */
/* #define VOID */
/*
 *  other values that can be tailored
 */
#define    MAXKEYSIZE   32             /* max # of chars in key (less null)  */
#define    NUMINDEXS    16             /* max # of open files (key and data) */
#define    MINKEYS      31             /* b-tree order                       */
/*
 *  end of stuff that should be changed
 */

/*
 *      General purpose Symbolic constants:
 */
#if !defined(OK) || ((OK) != 0)
#define OK 0            /* General purpose "no error" return value          */
#endif
#define NAMESIZE  80    /* maximum path/file name length                    */
#define SECSIZ 512      /* Sector size for header read/write calls          */
#define LOCAL static    /* f'ns local to this module                        */
#define ENTRY           /* entry points into this module                    */
#define MAXTREEHEIGHT 10   /* huge height 10 tree (10^10 keys)              */

/*
 *  structure for lioGet/PutCur(). These are currently undocumented
 *  entry points that set and restore BTree currency.
 */
struct currency {
        short sPtr;
        long stk[MAXTREEHEIGHT];
        char last[MAXKEYSIZE];
    };

/*
 *  #defines that may already be defined in stdio.h
 */

/*
 *  NULL is used as a non-success return code
 */
#ifndef NULL
#include    <stdio.h>
#endif

/*
 *  many BTree routines need a (long) NULL
 */
#ifndef LNULL
#define LNULL (long) NULL
#endif

/*
 *  file position equates for btFPos() (only used in isam)
 */
#ifndef EOF
#define EOF (-1)        /* end of file       */
#endif
#define BOF (-2)        /* beginning of file */
#define WHOKNOWS (-3)    /* unknown position  */

/*
 *  general "everything's OK" value
 */
#if !defined(GOOD) || ((GOOD) != 1)
#define GOOD 1
#endif

/*
 *  life threatening error value
 */
#if !defined(ERROR) || ((ERROR) != -1)
#define ERROR -1
#endif

/*
 *  true/false
 */
#ifndef TRUE
#define TRUE  1
#define FALSE 0
#endif
/*
 *        the t(ype) codes of a key value     (currently only string supported)
 */
#define    tSTRING       1          /* key is a string                   */
#define    tDECIMAL      2          /* key is floating point decimal     */
#define    tINTEGER      3          /* key is signed integer             */
#define    tUINTEGER     4          /* key is unsigned integer           */
#define    tLONG         5          /* key is signed long                */
#define    tULONG        6          /* key is unsigned long              */
#define    tSHORT        7          /* key size is only 16 bit integer   */
#define    tUSHORT       8          /* key size is only 16 bit integer   */
#define    tTRIPLE       9          /* key size is 24 bit unsigned int   */
#define    tDATA         10         /* file is actually data, not keys   */
#define    tUNKNOWN      11         /* unknown type of key               */

#define    zSTRING       -1
#define    zDECIMAL      (sizeof(double))
#define    zINTEGER      (sizeof(int))
#define    zUINTEGER     (sizeof(unsigned int))
#define    zLONG         (sizeof(long))
#define    zULONG        (sizeof(unsigned long))
#define    zSHORT        (sizeof(short))
#define    zUSHORT       (sizeof(unsigned short))
#define    zTRIPLE       3
#define    zDATA         -2
#define    zUNKNOWN      -3


/*
 *  (rarely needed) defines for the treeprint routine
 */
#define    NOTFOUND     0
#define    PREORDER     1              /* do an pre-order tree print        */
#define    INORDER      2              /* do an in-order tree print         */
/*
 *  end of btgconst.h
 */
