/*
 *  entry.h     All entry points for the BTree Library that any
 *              application should need. This file should be
 *              #included into all programs that call BTree routines
 */
extern void lioInit(void);   /* initialize the b-tree structures          */
                             /* this is called automatically now, but     */
                             /* remains here for compatability.           */

/* get size of key */
/* ~~~~~~~~~~~~~~~ */
extern int      lioSizeKey( int keyDes );

/* get type of key */
/* ~~~~~~~~~~~~~~~ */
extern int      lioTypeKey( int keyDes );

/* how many data records/keys are in the file */
/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
extern long     lioSize( int keyDes );

/* the number of the next available data rec */
/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
extern long     lioAvl( int keyDes );

/* close a data or index file */
/* ~~~~~~~~~~~~~~~~~~~~~~~~~~ */
extern int      lioClose( int keyDes );

/* create or test for the existance of a file */
/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
extern int      lioCreat( char *fName, int keySiz, int keyTyp, int dupAllowed );

/* signal that a data record is to be deleted */
/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
extern int      lioFree( int keyDes, long discard );

/* open a key or data file for processing */
/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
extern int      lioOpen( char *fName );

/* read a record from a data file */
/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
extern int      lioRead( int keyDes, long pageNo, char *dest );

/* write a record to a data file */
/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
extern int      lioWrite( int keyDes, long pageNo, char *src );

/* synchronize all files (flush buffers) */
/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
extern int      lioFlush( int keyDes );

/* add a key to an index file */
/* ~~~~~~~~~~~~~~~~~~~~~~~~~~ */
extern long     lioInsert( int keyDes, char *insKey, long insRcdNo );

/* search for a key in an index file */
/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
extern long     lioSearch( int keyDes, char *insKey );

/* search for the given key & record number */
/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
extern long     lioSerNum( int keyDes, char *key, long num );

/* remove a key from an index file */
/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
extern long     lioDelete( int keyDes, char *delKey );

/* find that key just < previous search key */
/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
extern long     lioPrev( int keyDes, char *foundKey );

/* find that key just > previous search key */
/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
extern long     lioNext( int keyDes, char *foundKey );

/* find the key <= a given search value */
/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
extern long     lioSrPrtl( int keyDes, char *srchKey, char *foundKey );

/* find the first key in a tree */
/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
extern long     lioFirst( int keyDes, char *fk );

/* find the last key in a tree */
/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
extern long     lioLast( int keyDes, char *fk );

/* return the record length of a file */
/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
extern int      lioPSize( int keyDes );

/* save BTree currency info */
/* ~~~~~~~~~~~~~~~~~~~~~~~~ */
extern void     lioGetCur( int keyDes, struct currency *p );

/* restore currency inf */
/* ~~~~~~~~~~~~~~~~~~~~ */
extern void     lioPutCur( int keyDes, struct currency *p );

/* check whether duplicates are allowed */
/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
extern int      lioDup( int keyDes );

extern int      ndxCmp1(int keyDes,char *left,char *right);



/*
 *  end of entry.h
 */
