#include        "include/Std.h"
#include        "include/Stack.h"

#include        "include/btgconst.h"
#include        "include/entry.h"
#include        "include/system.h"


typedef struct  _XL {
    FILE        *fdes;
    int         rlen;
} XL;

char            *strToKey(char *, char *, int, int, int );
char            *toKey( char *, char *, int, int );
char            *keyToStr( int, char *, char * );
XL              *xlOpen( char * );
int             xlRead( XL *, char *, long );
int             xlClose( XL * );
int             xlSearch( char ***, XL *, int, char * );

/*          */
/* ----------------------------------------------------------------- */

char *strToKey(char *rowo,char *rowi,int posfield,int lenfield,int typkey) {
    double          d;
    int             i;
    long            l;
    short           s;
    char            tmp[80];
    
    memcpy( rowo, rowi+posfield-1, lenfield );
    rowo[lenfield] = '\0';
    memset( tmp, ' ', sizeof(tmp) );
    if( !memcmp( tmp, rowo, lenfield ) ) return( (char *)NULL );
    switch( typkey ) {
        case tDECIMAL :
            d = atof( rowo );
            memcpy( rowo, (char *)&d, zDECIMAL );
            break;
        case tINTEGER :
            i = atoi( rowo );
            memcpy( rowo, (char *)&i, zINTEGER );
            break;
        case tLONG    :
            l = atol( rowo );
            memcpy( rowo, (char *)&l, zLONG );
            break;
        case tSHORT   :
            s = (short)atoi( rowo );
            memcpy( rowo, (char *)&s, zSHORT );
            break;
        case tUINTEGER :
            i = atoi( rowo );
            memcpy( rowo, (char *)&i, zUINTEGER );
            break;
        case tULONG    :
            l = atol( rowo );
            memcpy( rowo, (char *)&l, zULONG );
            break;
        case tUSHORT   :
            s = (short)atoi( rowo );
            memcpy( rowo, (char *)&s, zUSHORT );
            break;
        case tTRIPLE :
            l = atol( rowo );
            ltoc( l, rowo );
            break;
    }
    return( rowo );
}

char *toKey( char *rowo, char *rowi, int typkey, int lenkey ) {
    double          d;
    int             i;
    long            l;
    short           s;
    char            tmp[80];
    
    switch( typkey ) {
        case tSTRING :
            rjust( rowo, rowi, lenkey );
            break;
        case tDECIMAL :
            d = atof( rowi );
            memcpy( rowo, (char *)&d, zDECIMAL );
            break;
        case tINTEGER :
            i = atoi( rowi );
            memcpy( rowo, (char *)&i, zINTEGER );
            break;
        case tLONG    :
            l = atol( rowi );
            memcpy( rowo, (char *)&l, zLONG );
            break;
        case tSHORT   :
            s = (short)atoi( rowi );
            memcpy( rowo, (char *)&s, zSHORT );
            break;
        case tUINTEGER :
            i = atoi( rowi );
            memcpy( rowo, (char *)&i, zUINTEGER );
            break;
        case tULONG    :
            l = atol( rowi );
            memcpy( rowo, (char *)&l, zULONG );
            break;
        case tUSHORT   :
            s = (short)atoi( rowi );
            memcpy( rowo, (char *)&s, zUSHORT );
            break;
        case tTRIPLE :
            l = atol( rowi );
            ltoc( l, rowo );
            break;
    }
    return( rowo );
}

char *keyToStr( int keyDes, char *rowo, char *rowi ) {
    int         len;
    int         ii;
    long        ll;
    double      dd;
   
    switch( lioTypeKey( keyDes ) ) {
        case tSTRING :
            len = lioSizeKey( keyDes );
            sprintf( rowo, "%.*s", len, rowi );
        	break;

        case tDECIMAL :
            memcpy( &dd, rowi, zDECIMAL );
            sprintf( rowo, "%lf", dd );
            break;
            
        case tSHORT :
            ll = 0;
            memcpy( &ll, rowi, zSHORT );
            if( ll & 0x8000 ) ll |= 0xffff0000;
            sprintf( rowo, "%d", ll );
            break;
            
        case tUSHORT :
            ll = 0;
            memcpy( &ll, rowi, zUSHORT );
            sprintf( rowo, "%d", ll );
            break;
            
        case tINTEGER :
            ii = 0;
            memcpy( &ii, rowi, zINTEGER );
            sprintf( rowo, "%d", ii );
            break;
            
        case tUINTEGER :
            ii = 0;
            memcpy( &ii, rowi, zUINTEGER );
            sprintf( rowo, "%u", ii );
            break;
            
        case tLONG :
            ll = 0;
            memcpy( &ll, rowi, zLONG );
            sprintf( rowo, "%d", ll );
            break;
        
        case tULONG :
            ll = 0;
            memcpy( &ll, rowi, zULONG );
            sprintf( rowo, "%u", ll );
            break;
        
        case tTRIPLE :
            ll = ctol( rowi );
            sprintf( rowo, "%d", ll );
            break;
    }
    return( rowo );
}



/*    (  ) */
/* ------------------------------------------------ */

XL *xlOpen( char *fname ) {
    XL          *xl;
    char        tmp[1024];
    
    if( !(xl=(XL *)malloc( sizeof(XL) )) ) {
        put_message( "XL-memalloc error for `%s'!", fname );
        return( (XL *)NULL );
    }
    if( !(xl->fdes = fopen( fname, "r" )) ) {
        put_message( "Can't open data file `%s'!", fname );
        return( (XL *)NULL );
    }
    fgets( tmp, sizeof(tmp), xl->fdes );
    xl->rlen = strlen( tmp );
    rewind( xl->fdes );
    return( xl );
}

/*      */
/* -------------------------------- */

int xlRead( XL *xl, char *str, long recno ) {
    fseek( xl->fdes, (recno-1)*(xl->rlen), SEEK_SET );
    fgets( str, xl->rlen, xl->fdes );
    return( GOOD );
}

int xlClose( XL *xl ) {
    fclose( xl->fdes );
    free( xl );
    return( GOOD );
}

/*           */
/* ---------------------------------------------------- */

int xlSearch( char ***data, XL *xl, int keyDes, char *insKey ) {
    int             typkey, lenkey;
    long            recp = 0;
    long            recn = 0;
    char            keyp[80], keyn[80];
    char            rowi[1024];
    int             dountil = TRUE;

    STACK           top;
    STACK           *stk;
    int             nstk;

    char            **dt = (char **)NULL;
    
    stk = init_stack( &top, &nstk );

    typkey = lioTypeKey( keyDes );
    lenkey = lioSizeKey( keyDes );
    toKey( keyp, insKey, typkey, lenkey );
    toKey( keyn, insKey, typkey, lenkey );
    if( (recn = lioSearch( keyDes, keyn )) == ERROR ) {
        put_message( "xlSearch(): lioSearch() error for '%s'!\n", insKey );
        return( ERROR );
    }
    recp = recn;
    xlRead( xl, rowi, recn );
    if( !(stk=push_stack( stk, &nstk, rowi, strlen(rowi)+1 )) ) return( ERROR );
    while( dountil == TRUE ) {
        recn = lioNext( keyDes, keyn );
        switch( recn ) {
            case ERROR :
                put_message( "xlSearch(): lioNext() error for '%s'!\n", insKey );
                return( ERROR );
            case 0 :
                put_message( "xlSearch(): '%s' Not found!", insKey );
                return( UNKNOWN );
            default :
                if( recn == recp || ndxCmp1( keyDes, keyn, keyp ) > 0 ) {
                    dountil = FALSE;
                    break;
                }
                recp = recn;
                xlRead( xl, rowi, recp );
                if( !(stk=push_stack( stk, &nstk, rowi, strlen(rowi)+1 )) ) {
                    return( ERROR );
                }
                break;
        }
    }
    
    dt = popa_stack( stk, &nstk );
    *data = dt;
    return( GOOD );
}
